
/**
 * One flying piece of fruit for the Fruit Ninja game.
 * 
 * @author Jim Teresco 
 */
import objectdraw.*;
import java.awt.*;
import java.util.Random;

public class FlyingFruit extends ActiveObject {
    
    // lots of speeds and sizes
    private static final double MIN_XSPEED = -2;
    private static final double MAX_XSPEED = 2;
    private static final double MIN_YSPEED = -6;
    private static final double MAX_YSPEED = -10;
    private static final double FRUIT_SIZE = 40;
    private static final int SHRAPNEL_COUNT = 4;
    
    private static final double GRAVITY = 0.1;
    
    private static final int PAUSE_TIME = 33;
    
    // the actual graphical object that's flying
    private FilledOval fruit;
    
    // our speeds
    private double xSpeed, ySpeed;
    
    // need to remember the canvas
    private DrawingCanvas c;
    
    // reference to the game's WindowController which will be used
    // to report when new instances are created and when instances
    // fall off the bottom of the canvas
    private FruitNinja game;
    
    // have we hit this one yet?
    private boolean hit = false;
    
    // the constructor is responsible for remembering some information,
    // choosing a starting position and speed, and telling the game
    // that it exists.
    public FlyingFruit(FruitNinja game, DrawingCanvas c) {
        
        this.game = game;
        this.c = c;
        
        Random r = new Random();
        
        // random x position within middle "half" of canvas
        double xPos = r.nextDouble()*c.getWidth()/2 + c.getWidth()/4 - FRUIT_SIZE/2;
        
        // our actual fruit object
        fruit = new FilledOval(xPos, c.getHeight(), FRUIT_SIZE, FRUIT_SIZE, c);
        fruit.setColor(new Color(r.nextInt(200), r.nextInt(200), r.nextInt(200)));
        
        // calculate initial speed
        xSpeed = r.nextDouble()*(MAX_XSPEED - MIN_XSPEED) + MIN_XSPEED;
        ySpeed = r.nextDouble()*(MAX_YSPEED - MIN_YSPEED) + MIN_YSPEED;
        
        // tell the game that we have a new FlyingFruit
        game.newFruit(this);
        
        start();
    }
    
    // method to see if the fruit object here contains a point
    public boolean contains(Location point) {
        
        return fruit.contains(point);
    }

    // method to call when this fruit object has been hit
    public void hit() {
        
        hit = true;
    }
    
    // the life of a FlyingFruit
    public void run() {

        boolean done = false;
        
        while (!done) {
            
            // move it
            fruit.move(xSpeed, ySpeed);
            // acceleration due to gravity
            ySpeed += GRAVITY;
            // wait a little
            pause(PAUSE_TIME);
            
            // see if we've gone off the bottom
            if (ySpeed > 0 && fruit.getY() > c.getHeight()) {
                done = true;
            }
            
            // if we've been hit, explode!
            if (hit) {
                done = true;
                // start the shrapnel here
                Color color = fruit.getColor();
                double x = fruit.getX() + FRUIT_SIZE/2;
                double y = fruit.getY() + FRUIT_SIZE/2;
                for (int i = 0; i < SHRAPNEL_COUNT; i++) {
                    new Shrapnel(x, y, xSpeed, ySpeed, color, c);
                }
            }
        }
        
        // inform the game that this object is done, hit value indicates
        // hit or miss so game can update status
        game.fruitDone(this, hit);
        
        fruit.removeFromCanvas();
    }
}
